package sun.dc.pr;

import sun.java2d.DisposerRecord;
import sun.dc.DuctusRenderingEngine;
import sun.dc.path.FastPathProducer;
import sun.dc.path.PathException;
import sun.dc.path.PathError;
import sun.java2d.Disposer;
import sun.dc.path.PathConsumer;
import sun.java2d.pipe.AATileGenerator;

public class Rasterizer implements AATileGenerator
{
    public static final int EOFILL = 1;
    public static final int NZFILL = 2;
    public static final int STROKE = 3;
    public static final int ROUND = 10;
    public static final int SQUARE = 20;
    public static final int BUTT = 30;
    public static final int BEVEL = 40;
    public static final int MITER = 50;
    public static final int TILE_SIZE;
    public static final int TILE_SIZE_L2S;
    public static final int MAX_ALPHA = 1000000;
    public static final int MAX_MITER = 10;
    public static final int MAX_WN = 63;
    public static final int TILE_IS_ALL_0 = 0;
    public static final int TILE_IS_ALL_1 = 1;
    public static final int TILE_IS_GENERAL = 2;
    private static final int BEG = 1;
    private static final int PAC_FILL = 2;
    private static final int PAC_STROKE = 3;
    private static final int PATH = 4;
    private static final int SUBPATH = 5;
    private static final int RAS = 6;
    private int state;
    private PathFiller filler;
    private PathStroker stroker;
    private PathDasher dasher;
    private PathConsumer curPC;
    
    public Rasterizer() {
        this.state = 1;
        this.filler = new PathFiller();
        this.stroker = new PathStroker(this.filler);
        this.dasher = new PathDasher(this.stroker);
        Disposer.addRecord(this, new Rasterizer.ConsumerDisposer(this.filler, this.stroker, this.dasher));
    }
    
    public void setUsage(int n) throws PRError {
        if (this.state != 1) {
            throw new PRError("setUsage: unexpected");
        }
        if (n == 1) {
            this.filler.setFillMode(1);
            this.curPC = this.filler;
            this.state = 2;
        }
        else if (n == 2) {
            this.filler.setFillMode(2);
            this.curPC = this.filler;
            this.state = 2;
        }
        else if (n == 3) {
            this.curPC = this.stroker;
            this.filler.setFillMode(2);
            this.stroker.setPenDiameter(1.0f);
            this.stroker.setPenT4((float[])null);
            this.stroker.setCaps(10);
            this.stroker.setCorners(10, 0.0f);
            this.state = 3;
        }
        else {
            throw new PRError("setUsage: unknown usage type");
        }
        return;
    }
    
    public void setPenDiameter(float penDiameter) throws PRError {
        if (this.state != 3) {
            throw new PRError("setPenDiameter: unexpected");
        }
        this.stroker.setPenDiameter(penDiameter);
        return;
    }
    
    public void setPenT4(float[] penT4) throws PRError {
        if (this.state != 3) {
            throw new PRError("setPenT4: unexpected");
        }
        this.stroker.setPenT4(penT4);
        return;
    }
    
    public void setPenFitting(float n, int n2) throws PRError {
        if (this.state != 3) {
            throw new PRError("setPenFitting: unexpected");
        }
        this.stroker.setPenFitting(n, n2);
        return;
    }
    
    public void setPenDisplacement(float n, float n2) throws PRError {
        if (this.state != 3) {
            throw new PRError("setPenDisplacement: unexpected");
        }
        this.stroker.setOutputT6(new float[] { 1.0f, 0.0f, 0.0f, 1.0f, n, n2 });
        return;
    }
    
    public void setCaps(int caps) throws PRError {
        if (this.state != 3) {
            throw new PRError("setCaps: unexpected");
        }
        this.stroker.setCaps(caps);
        return;
    }
    
    public void setCorners(int n, float n2) throws PRError {
        if (this.state != 3) {
            throw new PRError("setCorners: unexpected");
        }
        this.stroker.setCorners(n, n2);
        return;
    }
    
    public void setDash(float[] array, float n) throws PRError {
        if (this.state != 3) {
            throw new PRError("setDash: unexpected");
        }
        this.dasher.setDash(array, n);
        this.curPC = this.dasher;
        return;
    }
    
    public void setDashT4(float[] dashT4) throws PRError {
        if (this.state != 3) {
            throw new PRError("setDashT4: unexpected");
        }
        this.dasher.setDashT4(dashT4);
        return;
    }
    
    public void beginPath(float[] array) throws PRError {
        this.beginPath();
    }
    
    public void beginPath() throws PRError {
        if (this.state != 2 && this.state != 3) {
            throw new PRError("beginPath: unexpected");
        }
        try {
            this.curPC.beginPath();
            this.state = 4;
        }
        catch (PathError pathError) {
            throw new PRError(pathError.getMessage());
        }
        return;
    }
    
    public void beginSubpath(float n, float n2) throws PRError {
        if (this.state != 4 && this.state != 5) {
            throw new PRError("beginSubpath: unexpected");
        }
        try {
            this.curPC.beginSubpath(n, n2);
            this.state = 5;
        }
        catch (PathError pathError) {
            throw new PRError(pathError.getMessage());
        }
        return;
    }
    
    public void appendLine(float n, float n2) throws PRError {
        if (this.state != 5) {
            throw new PRError("appendLine: unexpected");
        }
        try {
            this.curPC.appendLine(n, n2);
        }
        catch (PathError pathError) {
            throw new PRError(pathError.getMessage());
        }
        return;
    }
    
    public void appendQuadratic(float n, float n2, float n3, float n4) throws PRError {
        if (this.state != 5) {
            throw new PRError("appendQuadratic: unexpected");
        }
        try {
            this.curPC.appendQuadratic(n, n2, n3, n4);
        }
        catch (PathError pathError) {
            throw new PRError(pathError.getMessage());
        }
        return;
    }
    
    public void appendCubic(float n, float n2, float n3, float n4, float n5, float n6) throws PRError {
        if (this.state != 5) {
            throw new PRError("appendCubic: unexpected");
        }
        try {
            this.curPC.appendCubic(n, n2, n3, n4, n5, n6);
        }
        catch (PathError pathError) {
            throw new PRError(pathError.getMessage());
        }
        return;
    }
    
    public void closedSubpath() throws PRError {
        if (this.state != 5) {
            throw new PRError("closedSubpath: unexpected");
        }
        try {
            this.curPC.closedSubpath();
        }
        catch (PathError pathError) {
            throw new PRError(pathError.getMessage());
        }
        return;
    }
    
    public void endPath() throws PRError, PRException {
        if (this.state != 4 && this.state != 5) {
            throw new PRError("endPath: unexpected");
        }
        try {
            this.curPC.endPath();
            this.state = 6;
        }
        catch (PathError pathError) {
            throw new PRError(pathError.getMessage());
        }
        catch (PathException ex) {
            throw new PRException(ex.getMessage());
        }
        return;
    }
    
    public void useProxy(FastPathProducer fastPathProducer) throws PRError, PRException {
        if (this.state != 2 && this.state != 3) {
            throw new PRError("useProxy: unexpected");
        }
        try {
            this.curPC.useProxy(fastPathProducer);
            this.state = 6;
        }
        catch (PathError pathError) {
            throw new PRError(pathError.getMessage());
        }
        catch (PathException ex) {
            throw new PRException(ex.getMessage());
        }
        return;
    }
    
    public void getAlphaBox(int[] array) throws PRError {
        this.filler.getAlphaBox(array);
    }
    
    public void setOutputArea(float n, float n2, int n3, int n4) throws PRError, PRException {
        this.filler.setOutputArea(n, n2, n3, n4);
    }
    
    public int getTileState() throws PRError {
        return this.filler.getTileState();
    }
    
    public void writeAlpha(byte[] array, int n, int n2, int n3) throws PRError, PRException, InterruptedException {
        this.filler.writeAlpha(array, n, n2, n3);
    }
    
    public void writeAlpha(char[] array, int n, int n2, int n3) throws PRError, PRException, InterruptedException {
        this.filler.writeAlpha(array, n, n2, n3);
    }
    
    @Override
    public void nextTile() throws PRError {
        this.filler.nextTile();
    }
    
    public void reset() {
        this.state = 1;
        this.filler.reset();
        this.stroker.reset();
        this.dasher.reset();
    }
    
    @Override
    public int getTileWidth() {
        return Rasterizer.TILE_SIZE;
    }
    
    @Override
    public int getTileHeight() {
        return Rasterizer.TILE_SIZE;
    }
    
    @Override
    public int getTypicalAlpha() {
        int tileState = this.filler.getTileState();
        switch (tileState) {
            case 0: {
                tileState = 0;
                break;
            }
            case 1: {
                tileState = 255;
                break;
            }
            case 2: {
                tileState = 128;
                break;
            }
        }
        return tileState;
    }
    
    @Override
    public void getAlpha(byte[] array, int n, int n2) {
        synchronized (Rasterizer.class) {
            try {
                this.filler.writeAlpha(array, 1, n2, n);
            }
            catch (PRException ex) {
                throw new InternalError("Ductus AA error: " + ex.getMessage());
            }
            catch (InterruptedException ex2) {
                Thread.currentThread().interrupt();
            }
        }
    }
    
    @Override
    public void dispose() {
        DuctusRenderingEngine.dropRasterizer(this);
    }
    
    static {
        TILE_SIZE = 1 << PathFiller.tileSizeL2S;
        TILE_SIZE_L2S = PathFiller.tileSizeL2S;
    }
    
    private static class ConsumerDisposer implements DisposerRecord
    {
        PathConsumer filler;
        PathConsumer stroker;
        PathConsumer dasher;
        
        public ConsumerDisposer(PathConsumer filler, PathConsumer stroker, PathConsumer dasher) {
            this.filler = filler;
            this.stroker = stroker;
            this.dasher = dasher;
        }
        
        @Override
        public void dispose() {
            this.filler.dispose();
            this.stroker.dispose();
            this.dasher.dispose();
        }
    }
}
